package com.newriders.jxta.chapter11.chat;

import com.newriders.jxta.chapter11.protocol.InitiateChatRequestMessage;
import com.newriders.jxta.chapter11.protocol.InitiateChatResponseMessage;


/**
 * An interface to encapsulate an object that listens for notification
 * from the ChatService of newly arrived requests for a chat session and
 * responds to requests for a chat session.
 */
public interface ChatListener
{
    /**
     * Notify the listener that a chat session has been approved.
     *
     * @param   response the response to the request for a chat session.
     */
    public void chatApproved(InitiateChatResponseMessage response);

    /**
     * Notify the listener that a chat session has been requested.
     *
     * @param   request the object containing the chat session request info.
     * @param   queryID the query ID from the Resolver Query Message used to
     *         send the request.
     */
    public void chatRequested(InitiateChatRequestMessage request, 
        int queryID);
}

