package com.newriders.jxta.chapter10;

import java.util.Enumeration;

import net.jxta.discovery.DiscoveryService;

import net.jxta.exception.PeerGroupException;

import net.jxta.id.IDFactory;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.protocol.PeerGroupAdvertisement;


/**
 * Create a new peer group and publish it.
 */
public class CreatePeerGroup
{
    /**
     * The Net Peer Group for the application.
     */
    private PeerGroup netPeerGroup = null;

 
    /**
     * Creates a new peer group using the Net Peer Group's
     * Module Implementation Advertisement as a template.
     *
     * @exception   Exception if an error occurs retrieving the
     *              copy of the Net Peer Group's Module
     *              Implementation Advertisement.
     */
    public void createPeerGroup() throws Exception
    {
        // The name and description for the peer group.
        String name = "CreatePeerGroup";
        String description = 
            "An example peer group to test peer group creation";

        // Obtain a preformed ModuleImplAdvertisement to 
        // use when creating the new peer group.
        ModuleImplAdvertisement implAdv =
            netPeerGroup.getAllPurposePeerGroupImplAdvertisement();

        // Create the Peer Group ID.
        PeerGroupID groupID = IDFactory.newPeerGroupID();

        // Create the new group using the Peer Group ID,
        // advertisement, name, and description.
        PeerGroup newGroup = netPeerGroup.newGroup(
            groupID, implAdv, name, description);

        // Need to publish the group remotely only because
        // newGroup() handles publishing to the local peer.
        PeerGroupAdvertisement groupAdv = 
            newGroup.getPeerGroupAdvertisement();
        DiscoveryService discovery = 
            netPeerGroup.getDiscoveryService();
        discovery.remotePublish(groupAdv,
            DiscoveryService.GROUP);
    }

    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform
     *             can't be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        netPeerGroup = PeerGroupFactory.newNetPeerGroup();
    }

    /**
     * Runs the application.
     *
     * @param   args the command-line arguments passed to 
     *     the application.
     */
    public static void main(String[] args)
    {
        CreatePeerGroup creator = new CreatePeerGroup();

        try
        {
            // Initialize the JXTA platform.
            creator.initializeJXTA();

            // Create the group.
            creator.createPeerGroup();

            // Exit.
            System.exit(0);
        }
        catch (Exception e)
        {
            System.out.println("Error starting JXTA platform: " 
                + e);
            System.exit(1);
        }
    }
}