package com.newriders.jxta.chapter10;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import java.net.MalformedURLException;
import java.net.UnknownServiceException;
import java.net.URL;

import java.util.Enumeration;
import java.util.Hashtable;

import net.jxta.discovery.DiscoveryService;

import net.jxta.document.AdvertisementFactory;
import net.jxta.document.Element;
import net.jxta.document.MimeMediaType;
import net.jxta.document.StructuredDocument;
import net.jxta.document.StructuredDocumentFactory;
import net.jxta.document.StructuredTextDocument;

import net.jxta.exception.PeerGroupException;
import net.jxta.exception.ServiceNotFoundException;

import net.jxta.id.IDFactory;

import net.jxta.impl.peergroup.StdPeerGroupParamAdv;

import net.jxta.peergroup.PeerGroup;
import net.jxta.peergroup.PeerGroupID;
import net.jxta.peergroup.PeerGroupFactory;

import net.jxta.platform.ModuleClassID;
import net.jxta.platform.ModuleSpecID;

import net.jxta.protocol.ModuleClassAdvertisement;
import net.jxta.protocol.ModuleImplAdvertisement;
import net.jxta.protocol.ModuleSpecAdvertisement;
import net.jxta.protocol.PeerGroupAdvertisement;


/**
 * An application to create a peer group, configure a new service for 
 * the peer group, and then interact with other peers using that new 
 * service.
 */
public class ExampleServiceTest implements ExampleServiceListener
{
    /**
     * The Module Class ID to use for the service.
     * YOU SHOULD REPLACE THIS WITH ONE YOU GENERATE 
     * YOURSELF USING THE GenerateID APPLICATION!
     */
    private static final String refModuleClassID = 
        "urn:jxta:uuid-128E938121DD4957B74B90EE27FDC61F05";

    /**
     * The Module Specification ID to use for the service.
     * YOU SHOULD REPLACE THIS WITH ONE YOU GENERATE 
     * YOURSELF USING THE GenerateID APPLICATION!
     */
    private static final String refModuleSpecID = 
        "urn:jxta:uuid-128E938121DD4957B74B90EE27FDC61FA385BCB"
        + "1BA504B0FA69F99FE84CDC25B06";

    /**
     * The Peer Group ID to use for the application.
     * YOU SHOULD REPLACE THIS WITH ONE YOU GENERATE 
     * YOURSELF USING THE GenerateID APPLICATION!
     */
    private static final String refPeerGroupID = 
        "urn:jxta:uuid-A87A7DD0762F47E88B2FB5452D47B3A802";

    /**
     * The peer group Module Specification ID to use for the application.
     * YOU SHOULD REPLACE THIS WITH ONE YOU GENERATE 
     * YOURSELF USING THE GenerateID APPLICATION!
     */
    private static final String refPeerGroupSpec = 
        "urn:jxta:uuid-DEADBEEFDEAFBABAFEEDBABE00000001CB18295"
        + "F0DE94F99983AA2F00C1DE42F06";
    
    /**
     * The Net Peer Group for the application.
     */
    private PeerGroup netPeerGroup = null;

    /**
     * The frame for the application user interface.
     */
    private JFrame clientFrame = new JFrame("Exponentiator");

    /**
     * The textfield for accepting the base input for the 
     * exponentiation operation.
     */
    private JTextField baseText = new JTextField(5);

    /**
     * The textfield for accepting the power input for the 
     * exponentiation operation.
     */
    private JTextField powerText = new JTextField(5);

    /**
     * The new group created by the application.
     */
    private PeerGroup newGroup = null;


    /**
     * Create the Module Class Advertisement for the service, using the 
     * preconfigured ID in refModuleClassID.
     *
     * @return      the generated Module Class Advertisement.
     * @exception   UnknownServiceException, MalformedURLException thrown 
     *              if the refModuleClassID is invalid or malformed.
     */
    private ModuleClassAdvertisement createModuleClassAdv() 
        throws UnknownServiceException, MalformedURLException
    {
        // Create the class ID from the refModuleClassID string.
        ModuleClassID classID = (ModuleClassID) IDFactory.fromURL(
            new URL((refModuleClassID)));

        // Create the Module Class Advertisement.
        ModuleClassAdvertisement moduleClassAdv = 
            (ModuleClassAdvertisement)                        AdvertisementFactory.newAdvertisement(
                    ModuleClassAdvertisement.getAdvertisementType());

        // Configure the Module Class Advertisement.
        moduleClassAdv.setDescription(
            "A service to handle exponentiation math problems.");
        moduleClassAdv.setModuleClassID(classID);
        moduleClassAdv.setName("Exponentiator Class");

        // Return the advertisement to the caller.
        return moduleClassAdv;
    }

    /**
     * Create the Module Implementation Advertisement for the service, 
     * using the specification ID in the passed in ModuleSpecAdvertisement
     * advertisement. Use the given ModuleImplAdvertisement to create the
     * compatibility element of the module impl specification.
     *
     * @param       groupImpl the ModuleImplAdvertisement of the parent 
     *              peer group.
     * @param       moduleSpecAdv the source of the specification ID.
     * @return      the generated Module Implementation Advertisement.
     */
    private ModuleImplAdvertisement createModuleImplAdv(
        ModuleImplAdvertisement groupImpl, 
        ModuleSpecAdvertisement moduleSpecAdv)
    {
        // Get the specification ID from the passed advertisement.
        ModuleSpecID specID = moduleSpecAdv.getModuleSpecID();

        // Create the Module Implementation Advertisement.
        ModuleImplAdvertisement moduleImplAdv = (ModuleImplAdvertisement) 
            AdvertisementFactory.newAdvertisement(
                ModuleImplAdvertisement.getAdvertisementType());

        // Configure the Module Implementation Advertisement.
        moduleImplAdv.setCode(
            "com.newriders.jxta.chapter10.ExampleServiceImpl");
        moduleImplAdv.setCompat(groupImpl.getCompat());
        moduleImplAdv.setDescription(
            "Reference Exponentiator implementation");
        moduleImplAdv.setModuleSpecID(specID);
        moduleImplAdv.setProvider("Brendon J. Wilson");

        // Return the advertisement to the caller.
        return moduleImplAdv;
    }

    /**
     * Create the Module Specification Advertisement for the service, 
     * using the preconfigured ID in refModuleSpecID.
     *
     * @return      the generated Module Class Advertisement.
     * @exception   UnknownServiceException, MalformedURLException thrown 
     *              if the refModuleSpecID is invalid or malformed.
     */
    private ModuleSpecAdvertisement createModuleSpecAdv() 
        throws UnknownServiceException, MalformedURLException
    {
        // Create the specification ID from the refModuleSpecID string.
        ModuleSpecID specID = (ModuleSpecID) IDFactory.fromURL(
            new URL((refModuleSpecID)));

        // Create the Module Specification Advertisement.
        ModuleSpecAdvertisement moduleSpecAdv = (ModuleSpecAdvertisement)
            AdvertisementFactory.newAdvertisement(
                ModuleSpecAdvertisement.getAdvertisementType());

        // Configure the Module Specification Advertisement.
        moduleSpecAdv.setCreator("Brendon J. Wilson");
        moduleSpecAdv.setDescription(
            "A specification for an exponentiation service.");
        moduleSpecAdv.setModuleSpecID(specID);
        moduleSpecAdv.setName("Exponentiator Spec");
        moduleSpecAdv.setSpecURI(
            "http://www.brendonwilson.com/projects/jxta");
        moduleSpecAdv.setVersion("1.0");

        // Return the advertisement to the caller.
        return moduleSpecAdv;
    }

    /**
     * Creates a peer group and configures the ExampleService 
     * implementation to run as a peer group service.
     *
     * @exception   Exception, PeerGroupException if there is a problem 
     *             while creating the peer group or the service 
     *             advertisements.
     */
    public void createPeerGroup() throws Exception, PeerGroupException
    {
        // The name and description for the peer group.
        String name = "CreatePeerGroup";
        String description = 
            "An example peer group to test peer group creation";

        // The Discovery service to use to publish the module and peer 
        // group advertisements.
        DiscoveryService discovery = netPeerGroup.getDiscoveryService();

        // Obtain a preformed ModuleImplAdvertisement to use when creating
        // the new peer group. This is the Module Implementation 
        // Advertisement of the Net Peer Group and contains all of the
        // services and applications already configured to run in that peer
        // group. Using this method simplifies the task of creating a new 
        // peer group and configuring a new service.
        ModuleImplAdvertisement implAdv = 
            netPeerGroup.getAllPurposePeerGroupImplAdvertisement();

        // Create the Module Class Advertisement.
        ModuleClassAdvertisement moduleClassAdv = createModuleClassAdv();

        // Create the Module Specification Advertisement.
        ModuleSpecAdvertisement moduleSpecAdv = createModuleSpecAdv();

        // Create the Module Implementation Advertisement.
        ModuleImplAdvertisement moduleImplAdv = 
            createModuleImplAdv(implAdv,moduleSpecAdv);

        // Get the parameters for the peer group's Module Implementation 
        // Advertisement to add our service.
        StdPeerGroupParamAdv params = 
            new StdPeerGroupParamAdv(implAdv.getParam());

        // Get the services from the parameters.
        Hashtable services = params.getServices();

        // Add our service to the set of services.
        services.put(moduleClassAdv.getModuleClassID(), moduleImplAdv);

        // Set the services on the parameters, and set the parameters on 
        // the implementation advertisement.
        params.setServices(services);
        implAdv.setParam((StructuredDocument) params.getDocument(
            new MimeMediaType("text", "xml")));

        // VERY IMPORTANT! You must change the Module Specification ID 
        // for the implementation advertisement. If you don't, the new 
        // peer group's Module Specification ID will still point to the 
        // old specification, and the new service will not be loaded.
        implAdv.setModuleSpecID((ModuleSpecID) IDFactory.fromURL(
            new URL(refPeerGroupSpec)));

        // Publish the Module Class and Specification Advertisements.
        discovery.publish(moduleClassAdv, DiscoveryService.ADV);
        discovery.remotePublish(moduleClassAdv, DiscoveryService.ADV);
        discovery.publish(moduleSpecAdv, DiscoveryService.ADV);
        discovery.remotePublish(moduleSpecAdv, DiscoveryService.ADV);
        discovery.publish(implAdv, DiscoveryService.ADV);
        discovery.remotePublish(implAdv, DiscoveryService.ADV);

        // Create the Peer Group ID.
        PeerGroupID groupID = (PeerGroupID) IDFactory.fromURL(
            new URL((refPeerGroupID)));

        // Create the new group using the group ID, advertisement, name, 
        // and description.
        newGroup = netPeerGroup.newGroup(groupID, implAdv, name,
            description);

        // Need to publish the group remotely only because newGroup() 
        // handles publishing to the local peer.
        PeerGroupAdvertisement groupAdv = 
            newGroup.getPeerGroupAdvertisement();
        discovery.remotePublish(groupAdv, DiscoveryService.GROUP);
    }

    /**
     * Starts the JXTA platform.
     *
     * @exception   PeerGroupException thrown if the platform can't 
     *              be started.
     */
    public void initializeJXTA() throws PeerGroupException
    {
        netPeerGroup = PeerGroupFactory.newNetPeerGroup();
    }

    /**
     * Starts the application.
     *
     * @param   args the command-line arguments passed to the application.
     */
    public static void main(String[] args)
    {
        ExampleServiceTest test = new ExampleServiceTest();

        try
        {
            // Initialize the JXTA platform.
            test.initializeJXTA();

            // Create the group.
            test.createPeerGroup();

            // Show a GUI to accept input.
            test.showGUI();
        }
        catch (Exception e)
        {
            System.out.println("Error starting JXTA platform: " + e);
            System.exit(1);
        }
    }

    /**
     * The implementation of the ExampleServiceListener interface. This
     * allows us to display a message each time a message is received by
     * the ExampleService.
     *
     * @param   answer the event containing the newly arrived message.
     */
    public void processAnswer(ExampleServiceEvent event)
    {
        // Extract the response message from the event object.
        ExampleResponseMsg er = event.getResponse();

        // Print out the answer given in the response.
        String answer = "The value of " + er.getBase() + " raised to " 
            + er.getPower() + " is: " + er.getAnswer(); 
        JOptionPane.showMessageDialog(null, answer, "Answer Received!",
            JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Convenience method to find the service and use it to send
     * a query to other peers' ExampleService.
     *
     * @param   base the base for the exponentiation query.
     * @param   power the power for the exponentiation query.
     */
    private void sendMessage(String base, String power)
    {
        try
        {
            // Convert the input to numbers.
            double baseValue = Double.parseDouble(base);
            double powerValue = Double.parseDouble(power);

            // Find the service on the peer group.
            ModuleClassID classID = (ModuleClassID) IDFactory.fromURL(
                new URL((refModuleClassID)));
            ExampleService exponentiator = 
                (ExampleService) newGroup.lookupService(classID);
            exponentiator.findAnswer(baseValue, powerValue);
        }
        catch (NumberFormatException e)
        {
            // Warn the user.
            JOptionPane.showMessageDialog(null, "The base and power must "
                + "both be numbers!", "Input Error!",
                JOptionPane.ERROR_MESSAGE);
        }
        catch (Exception e2)
        {
            // Warn the user.
            JOptionPane.showMessageDialog(null, "Error finding service!", 
                "Service Error!", JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Displays a user interface to allow the user to send queries to 
     * other peers.
     *
     * @exception   exceptions thrown only if the new service can't be 
     *             found.
     */
    private void showGUI() throws UnknownServiceException, 
        MalformedURLException, ServiceNotFoundException
    {
        JButton sendButton = new JButton("Send Message");
        JButton quitButton = new JButton("Quit");
        JPanel sendPane = new JPanel();
        JLabel baseLabel = new JLabel("Base:");
        JLabel powerLabel = new JLabel("Power:");
        Container pane = clientFrame.getContentPane();

 
        // Populate the GUI frame.
        sendPane.setLayout(new FlowLayout());
        sendPane.add(baseLabel);
        sendPane.add(baseText);
        sendPane.add(powerLabel);
        sendPane.add(powerText);
        sendPane.add(sendButton);
        sendPane.add(quitButton);
        pane.setLayout(new BorderLayout());
        pane.add(sendPane, BorderLayout.SOUTH);

        // Set up listeners for the buttons.
        sendButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e)
                {
                    // Send the message.
                    sendMessage(baseText.getText(), powerText.getText());

                    // Clear the text.
                    baseText.setText("");
                    powerText.setText("");
                }
            }
        );

        quitButton.addActionListener(
            new ActionListener()
            {
                public void actionPerformed(ActionEvent e)
                {
                    clientFrame.hide();
                    
                    // Stop the JXTA platform. Currently, there isn't any 
                    // nice way to do this.
                    System.exit(0);
                }
            }
        );

        // Find the new service on the peer group and add ourselves 
        // as a listener.
        ModuleClassID classID = (ModuleClassID) IDFactory.fromURL(
            new URL((refModuleClassID)));
        ExampleService exponentiator = 
            (ExampleService) newGroup.lookupService(classID);
        exponentiator.addListener(this);

        // Pack and display the user interface.
        clientFrame.pack();
        clientFrame.show();
    }
}