package net.jxta.impl.shell.bin.example4_3;

import java.io.IOException;

import net.jxta.discovery.DiscoveryService;

import net.jxta.peergroup.PeerGroup;

import net.jxta.impl.shell.GetOpt;
import net.jxta.impl.shell.ShellApp;
import net.jxta.impl.shell.ShellEnv;
import net.jxta.impl.shell.ShellObject;


/**
 * A simple example shell application to enable a user to send remote
 * discovery queries using the current peer group's Discovery service.
 */
public class example4_3 extends ShellApp
{
    /**
     * The type of advertisement to discover. Defaults to peer
     * advertisements.
     */
    private int type = DiscoveryService.PEER;

    /**
     * The maximum number of responses requested.
     */
    private int threshold = 10;

    /**
     * The Discovery service being used to discover advertisements.
     */
    private DiscoveryService discovery = null;


    /**
     * Parses the command-line arguments and initializes the command
     *
     * @param       args the arguments to be parsed.
     * @exception   IllegalArgumentException if an invalid parameter 
     *              is passed.
     */
    private void parseArguments(String[] args) 
        throws IllegalArgumentException
    {
        int option;

        // Parse the arguments to the command.
        GetOpt parser = new GetOpt(args, "a:t:");

        while ((option = parser.getNextOption()) != -1)
        {
            switch (option)
            {
                case 'a' :
                {
                    // Set the type of advertisement to discover.
                    type = Integer.parseInt(parser.getOptionArg());

                    // Validate the type.
                    if ((type < 0) || (type > 2))
                    {
                        // Default to the peer type.
                        type = DiscoveryService.PEER;
                    }

                    break;
                }

                case 't' :
                {
                    String argument = null;

                    if ((argument = parser.getOptionArg()) != null)
                    {
                        // Set the threshold.
                        threshold = Integer.parseInt(argument);
                    }

                    break;
                }
            }
        }
    }

    /**
     * Send a discovery request to remote peers via the Discovery service.
     *
     * @param   type the type of advertisement to discover.
     * @param   threshold the maximum number of advertisements to be
     *          returned by any single peer.
     */
    private void sendRemoteDiscovery(int type, int threshold)
    {
        discovery.getRemoteAdvertisements(null, type, null, null, 
            threshold, null);
    }

    /**
     * The implementation of the Shell command, invoked when the command 
     * is started by the user from the Shell.
     *
     * @param   args the command-line arguments passed to the command.
     * @return  a status code indicating the success or failure of 
     *          the command.
     */
    public int startApp(String[] args)
    {
        int result = appNoError;

        // Get the shell's environment.
        ShellEnv theEnvironment = getEnv();

        // Use the environment to obtain the current peer group.
        ShellObject theShellObject = theEnvironment.get("stdgroup");
        PeerGroup currentGroup = (PeerGroup) theShellObject.getObject();

        // Get the Discovery service for the current peer group.
        discovery = currentGroup.getDiscoveryService();

        try
        {
            // Parse the command-line arguments.
            parseArguments(args);
        }
        catch (IllegalArgumentException e)
        {
            println("Incorrect parameters passed to the command.");
            result = ShellApp.appParamError;
        }

        // Send a remote discovery request.
        sendRemoteDiscovery(type, threshold);

        return result;
    }
}

